/*-
 * Copyright (C)2014 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2014 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n\
@(#)V.M "VERS;
static const char rcsid[] = "$Id: ic.c,v 1.2 2014/08/31 06:34:46 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include "recobe.h"

#define RECOBE_TEXT_L 6

static const u_char
recobe_a2i[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0xB0, 0xB1, 0xB2, 0xB3,  0xB4, 0xB5, 0xB6, 0xB7,  0xB8, 0xB9, 0xBA, 0xBB,  0xBC, 0xBD, 0xBE, 0xBF
,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0X88, 0X89, 0X8A, 0X8B,  0X8C, 0X8D, 0X8E, 0X8F,  0X90, 0X91, 0X92, 0X93,  0X94, 0X95, 0X96, 0X97

,  0X98, 0X99, 0X9A, 0X81,  0X9C, 0X9D, 0X9E, 0X9F,  0XFF, 0X82, 0XA2, 0XA3,  0X87, 0XA5, 0XA6, 0X80
,  0XA8, 0XA9, 0XAA, 0XAB,  0XAC, 0XAD, 0X83, 0XAF,  0XFC, 0X84, 0XF2, 0XF3,  0X86, 0XF5, 0XF6, 0X85
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0xA1, 0xF1, 0xA4, 0xF4,  0xA7, 0xF7, 0xAE, 0xFE,  0XF8, 0XF9, 0XFA, 0XFB,  0XF0, 0XFD, 0X9B, 0xA0
};

static const u_char
recobe_i2a[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0XCF, 0XC3, 0XC9, 0XD6,  0XD9, 0XDF, 0XDC, 0XCC,  0XB0, 0XB1, 0XB2, 0XB3,  0XB4, 0XB5, 0XB6, 0XB7
,  0XB8, 0XB9, 0XBA, 0XBB,  0XBC, 0XBD, 0XBE, 0XBF,  0XC0, 0XC1, 0XC2, 0XFE,  0XC4, 0XC5, 0XC6, 0XC7
,  0xFF, 0xF0, 0XCA, 0XCB,  0xF2, 0XCD, 0XCE, 0xF4,  0XD0, 0XD1, 0XD2, 0XD3,  0XD4, 0XD5, 0xF6, 0XD7
,  0x80, 0x81, 0x82, 0x83,  0x84, 0x85, 0x86, 0x87,  0x88, 0x89, 0x8A, 0x8B,  0x8C, 0x8D, 0x8E, 0x8F

,  0x90, 0x91, 0x92, 0x93,  0x94, 0x95, 0x96, 0x97,  0x98, 0x99, 0x9A, 0x9B,  0x9C, 0x9D, 0x9E, 0x9F
,  0xA0, 0xA1, 0xA2, 0xA3,  0xA4, 0xA5, 0xA6, 0xA7,  0xA8, 0xA9, 0xAA, 0xAB,  0xAC, 0xAD, 0xAE, 0xAF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0xFC, 0xF1, 0XDA, 0XDB,  0xF3, 0XDD, 0XDE, 0xF5,  0XF8, 0XF9, 0XFA, 0XFB,  0XD8, 0XFD, 0xF7, 0xC8
};

static const u_char
recobe_i2k[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0X80, 0X81, 0X82, 0X83,  0X84, 0X85, 0X86, 0X87,  0X88, 0X89, 0X8A, 0X8B,  0X8C, 0X8D, 0X8E, 0X8F
,  0X90, 0X91, 0X92, 0X93,  0X94, 0X95, 0X96, 0X97,  0X98, 0X99, 0XA0, 0X9B,  0X9C, 0X9D, 0X9E, 0X9F
,  0x9A, 0xB3, 0XA2, 0XB1,  0XA4, 0XA5, 0XA6, 0XA7,  0XA8, 0XA9, 0XAA, 0XAB,  0XAC, 0XAD, 0XAE, 0XAF
,  0xE1, 0xE2, 0xF7, 0xE7,  0xE4, 0xE5, 0xF6, 0xFA,  0xE9, 0xEA, 0xEB, 0xEC,  0xED, 0xEE, 0xEF, 0xF0

,  0xF2, 0xF3, 0xF4, 0xF5,  0xE6, 0xE8, 0xE3, 0xFE,  0xFB, 0xFD, 0xFF, 0xF9,  0xF8, 0xFC, 0xE0, 0xF1
,  0xC1, 0xC2, 0xD7, 0xC7,  0xC4, 0xC5, 0xD6, 0xDA,  0xC9, 0xCA, 0xCB, 0xCC,  0xCD, 0xCE, 0xCF, 0xD0
,  0xD2, 0xD3, 0xD4, 0xD5,  0xC6, 0xC8, 0xC3, 0xDE,  0xDB, 0xDD, 0xDF, 0xD9,  0xD8, 0xDC, 0xC0, 0xD1
,  0XB0, 0xA3, 0XB2, 0XA1,  0XB4, 0XB5, 0XB6, 0XB7,  0XB8, 0XB9, 0XBA, 0XBB,  0XBC, 0XBD, 0XBE, 0XBF
};

static const u_char
recobe_k2i[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0X80, 0X81, 0X82, 0X83,  0X84, 0X85, 0X86, 0X87,  0X88, 0X89, 0X8A, 0X8B,  0X8C, 0X8D, 0X8E, 0X8F
,  0X90, 0X91, 0X92, 0X93,  0X94, 0X95, 0X96, 0X97,  0X98, 0X99, 0xA0, 0X9B,  0X9C, 0X9D, 0X9E, 0X9F
,  0X9A, 0XF3, 0XA2, 0xF1,  0XA4, 0XA5, 0XA6, 0XA7,  0XA8, 0XA9, 0XAA, 0XAB,  0XAC, 0XAD, 0XAE, 0XAF
,  0XF0, 0XA3, 0XF2, 0xA1,  0XF4, 0XF5, 0XF6, 0XF7,  0XF8, 0XF9, 0XFA, 0XFB,  0XFC, 0XFD, 0XFE, 0XFF

,  0xEE, 0xD0, 0xD1, 0xE6,  0xD4, 0xD5, 0xE4, 0xD3,  0xE5, 0xD8, 0xD9, 0xDA,  0xDB, 0xDC, 0xDD, 0xDE
,  0xDF, 0xEF, 0xE0, 0xE1,  0xE2, 0xE3, 0xD6, 0xD2,  0xEC, 0xEB, 0xD7, 0xE8,  0xED, 0xE9, 0xE7, 0xEA
,  0xCE, 0xB0, 0xB1, 0xC6,  0xB4, 0xB5, 0xC4, 0xB3,  0xC5, 0xB8, 0xB9, 0xBA,  0xBB, 0xBC, 0xBD, 0xBE
,  0xBF, 0xCF, 0xC0, 0xC1,  0xC2, 0xC3, 0xB6, 0xB2,  0xCC, 0xCB, 0xB7, 0xC8,  0xCD, 0xC9, 0xC7, 0xCA
};

static const u_char
recobe_a2k[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0xE1, 0xE2, 0xF7, 0xE7,  0xE4, 0xE5, 0xF6, 0xFA,  0xE9, 0xEA, 0xEB, 0xEC,  0xED, 0xEE, 0xEF, 0xF0
,  0xF2, 0xF3, 0xF4, 0xF5,  0xE6, 0xE8, 0xE3, 0xFE,  0xFB, 0xFD, 0xFF, 0xF9,  0xF8, 0xFC, 0xE0, 0xF1
,  0xC1, 0xC2, 0xD7, 0xC7,  0xC4, 0xC5, 0xD6, 0xDA,  0xC9, 0xCA, 0xCB, 0xCC,  0xCD, 0xCE, 0xCF, 0xD0
,  0x90, 0x91, 0x92, 0x81,  0x87, 0xB2, 0xB4, 0xA7,  0xA6, 0xB5, 0xA1, 0xA8,  0xAE, 0xAD, 0xAC, 0x83

,  0x84, 0x89, 0x88, 0x86,  0x80, 0x8A, 0xAF, 0xB0,  0xAB, 0xA5, 0xBB, 0xB8,  0xB1, 0xA0, 0xBE, 0xB9
,  0xBA, 0xB6, 0xB7, 0xAA,  0xA9, 0xA2, 0xA4, 0xBD,  0xBC, 0x85, 0x82, 0x8D,  0x8C, 0x8E, 0x8F, 0x8B
,  0xD2, 0xD3, 0xD4, 0xD5,  0xC6, 0xC8, 0xC3, 0xDE,  0xDB, 0xDD, 0xDF, 0xD9,  0xD8, 0xDC, 0xC0, 0xD1
,  0xB3, 0xA3, 0XBF, 0X93,  0X98, 0X97, 0X99, 0X9B,  0x9C, 0x95, 0x9E, 0x96,  0X9F, 0X9D, 0x94, 0x9A
};

static const u_char
recobe_k2a[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0xC4, 0xB3, 0xDA, 0xBF,  0xC0, 0xD9, 0xC3, 0xB4,  0xC2, 0xC1, 0xC5, 0xDF,  0xDC, 0xDB, 0xDD, 0xDE
,  0xB0, 0xB1, 0xB2, 0XF3,  0xFE, 0xF9, 0xFB, 0XF5,  0XF4, 0XF6, 0xFF, 0XF7,  0xF8, 0XFD, 0xFA, 0XFC
,  0xCD, 0xBA, 0xD5, 0xF1,  0xD6, 0xC9, 0xB8, 0xB7,  0xBB, 0xD4, 0xD3, 0xC8,  0xBE, 0xBD, 0xBC, 0xC6
,  0xC7, 0xCC, 0xB5, 0xF0,  0xB6, 0xB9, 0xD1, 0xD2,  0xCB, 0xCF, 0xD0, 0xCA,  0xD8, 0xD7, 0xCE, 0XF2

,  0xEE, 0xA0, 0xA1, 0xE6,  0xA4, 0xA5, 0xE4, 0xA3,  0xE5, 0xA8, 0xA9, 0xAA,  0xAB, 0xAC, 0xAD, 0xAE
,  0xAF, 0xEF, 0xE0, 0xE1,  0xE2, 0xE3, 0xA6, 0xA2,  0xEC, 0xEB, 0xA7, 0xE8,  0xED, 0xE9, 0xE7, 0xEA
,  0x9E, 0x80, 0x81, 0x96,  0x84, 0x85, 0x94, 0x83,  0x95, 0x88, 0x89, 0x8A,  0x8B, 0x8C, 0x8D, 0x8E
,  0x8F, 0x9F, 0x90, 0x91,  0x92, 0x93, 0x86, 0x82,  0x9C, 0x9B, 0x87, 0x98,  0x9D, 0x99, 0x97, 0x9A
};

static const u_char
recobe_a2w[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0X90, 0X91, 0X92, 0X81,  0X87, 0XB2, 0XB4, 0XA7,  0XA6, 0XB5, 0X99, 0XB3,  0XAE, 0XAD, 0XAC, 0X83

,  0X84, 0X89, 0X88, 0X86,  0X80, 0X8A, 0X98, 0X9C,  0XAB, 0XA5, 0XBB, 0XA3,  0XB1, 0X9A, 0XBE, 0X9F
,  0XA9, 0XB6, 0X9E, 0X93,  0X97, 0X9B, 0X9D, 0XBD,  0XBC, 0X85, 0X82, 0X8D,  0X8C, 0X8E, 0X8F, 0x8B
,  0xF0, 0xF1, 0xF2, 0xF3,  0xF4, 0xF5, 0xF6, 0xF7,  0xF8, 0xF9, 0xFA, 0xFB,  0xFC, 0xFD, 0xFE, 0xFF
,  0xA8, 0xB8, 0xAA, 0xBA,  0xAF, 0xBF, 0xA1, 0xA2,  0xB0, 0X95, 0xB7, 0X96,  0xB9, 0xA4, 0X94, 0xA0
};

static const u_char
recobe_w2a[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0XC4, 0XB3, 0XDA, 0XBF,  0XC0, 0XD9, 0XC3, 0XB4,  0XC2, 0XC1, 0XC5, 0XDF,  0XDC, 0XDB, 0XDD, 0XDE
,  0XB0, 0XB1, 0XB2, 0XD3,  0XFE, 0XF9, 0XFB, 0XD4,  0XC6, 0XBA, 0XCD, 0XD5,  0XC7, 0XD6, 0XD2, 0XCF
,  0xFF, 0xF6, 0xF7, 0XCB,  0xFD, 0XC9, 0XB8, 0XB7,  0xF0, 0XD0, 0xF2, 0XC8,  0XBE, 0XBD, 0XBC, 0xF4
,  0xF8, 0XCC, 0XB5, 0XBB,  0XB6, 0XB9, 0XD1, 0xFA,  0xF1, 0xFC, 0xF3, 0xCA,  0xD8, 0xD7, 0xCE, 0xF5

,  0x80, 0x81, 0x82, 0x83,  0x84, 0x85, 0x86, 0x87,  0x88, 0x89, 0x8A, 0x8B,  0x8C, 0x8D, 0x8E, 0x8F
,  0x90, 0x91, 0x92, 0x93,  0x94, 0x95, 0x96, 0x97,  0x98, 0x99, 0x9A, 0x9B,  0x9C, 0x9D, 0x9E, 0x9F
,  0xA0, 0xA1, 0xA2, 0xA3,  0xA4, 0xA5, 0xA6, 0xA7,  0xA8, 0xA9, 0xAA, 0xAB,  0xAC, 0xAD, 0xAE, 0xAF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
};

static const u_char
recobe_i2w[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0XB5, 0XAE, 0X82, 0XB6,  0X84, 0X85, 0X86, 0X87,  0X88, 0X89, 0XA9, 0X8B,  0XA4, 0XAC, 0XAB, 0XA5
,  0XA6, 0X91, 0X92, 0X93,  0X94, 0X95, 0X96, 0X97,  0X98, 0X99, 0XB0, 0X9B,  0XB4, 0XB1, 0XBB, 0XB7
,  0xA0, 0xA8, 0x80, 0x81,  0xAA, 0xBD, 0xB2, 0xAF,  0xA3, 0x8A, 0x8C, 0x8E,  0x8D, 0xAD, 0xA1, 0x8F
,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF

,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0xF0, 0xF1, 0xF2, 0xF3,  0xF4, 0xF5, 0xF6, 0xF7,  0xF8, 0xF9, 0xFA, 0xFB,  0xFC, 0xFD, 0xFE, 0xFF
,  0xB9, 0xB8, 0x90, 0x83,  0xBA, 0xBE, 0xB3, 0xBF,  0xBC, 0x9A, 0x9C, 0x9E,  0x9D, 0xA7, 0xA2, 0x9F
};

static const u_char
recobe_w2i[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0xA2, 0xA3, 0X82, 0xF3,  0X84, 0X85, 0X86, 0X87,  0X88, 0X89, 0xA9, 0X8B,  0xAA, 0xAC, 0xAB, 0xAF
,  0xF2, 0X91, 0X92, 0X93,  0X94, 0X95, 0X96, 0X97,  0X98, 0X99, 0xF9, 0X9B,  0xFA, 0xFC, 0xFB, 0xFF
,  0xA0, 0xAE, 0xFE, 0xA8,  0X8C, 0X8F, 0X90, 0xFD,  0xA1, 0X8A, 0xA4, 0X8E,  0X8D, 0xAD, 0X81, 0xA7
,  0X9A, 0X9D, 0xA6, 0xF6,  0X9C, 0X80, 0X83, 0X9F,  0xF1, 0xF0, 0xF4, 0X9E,  0xF8, 0xA5, 0xF5, 0xF7

,  0xB0, 0xB1, 0xB2, 0xB3,  0xB4, 0xB5, 0xB6, 0xB7,  0xB8, 0xB9, 0xBA, 0xBB,  0xBC, 0xBD, 0xBE, 0xBF
,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
};

static const u_char
recobe_k2w[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0X80, 0X81, 0X82, 0X83,  0X84, 0X85, 0X86, 0X87,  0X88, 0X89, 0X8A, 0X8B,  0X8C, 0X8D, 0X8E, 0X8F
,  0X90, 0X91, 0X92, 0X93,  0X94, 0X95, 0X96, 0X97,  0X98, 0X99, 0xA0, 0X9B,  0xB0, 0X9D, 0xB7, 0X9F
,  0X9A, 0XA1, 0XA2, 0xB8,  0XA4, 0XA5, 0XA6, 0XA7,  0XB3, 0XBF, 0XAA, 0XAB,  0XAC, 0XAD, 0XAE, 0XAF
,  0X9C, 0XB1, 0XB2, 0xA8,  0XB4, 0XB5, 0XB6, 0X9E,  0XA3, 0XB9, 0XBA, 0XBB,  0XBC, 0XBD, 0XBE, 0xA9

,  0xFE, 0xE0, 0xE1, 0xF6,  0xE4, 0xE5, 0xF4, 0xE3,  0xF5, 0xE8, 0xE9, 0xEA,  0xEB, 0xEC, 0xED, 0xEE
,  0xEF, 0xFF, 0xF0, 0xF1,  0xF2, 0xF3, 0xE6, 0xE2,  0xFC, 0xFB, 0xE7, 0xF8,  0xFD, 0xF9, 0xF7, 0xFA
,  0xDE, 0xC0, 0xC1, 0xD6,  0xC4, 0xC5, 0xD4, 0xC3,  0xD5, 0xC8, 0xC9, 0xCA,  0xCB, 0xCC, 0xCD, 0xCE
,  0xCF, 0xDF, 0xD0, 0xD1,  0xD2, 0xD3, 0xC6, 0xC2,  0xDC, 0xDB, 0xC7, 0xD8,  0xDD, 0xD9, 0xD7, 0xDA
};

static const u_char
recobe_w2k[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,  0X80, 0X81, 0X82, 0X83,  0X84, 0X85, 0X86, 0X87,  0X88, 0X89, 0X8A, 0X8B,  0X8C, 0X8D, 0X8E, 0X8F
,  0X90, 0X91, 0X92, 0X93,  0X94, 0X95, 0X96, 0X97,  0X98, 0X99, 0XA0, 0X9B,  0XB0, 0X9D, 0XB7, 0X9F
,  0x9A, 0XA1, 0XA2, 0XB8,  0XA4, 0XA5, 0XA6, 0XA7,  0xB3, 0xBF, 0XAA, 0XAB,  0XAC, 0XAD, 0XAE, 0XAF
,  0x9C, 0XB1, 0XB2, 0XA8,  0XB4, 0XB5, 0XB6, 0x9E,  0xA3, 0XB9, 0XBA, 0XBB,  0XBC, 0XBD, 0XBE, 0XA9

,  0xE1, 0xE2, 0xF7, 0xE7,  0xE4, 0xE5, 0xF6, 0xFA,  0xE9, 0xEA, 0xEB, 0xEC,  0xED, 0xEE, 0xEF, 0xF0
,  0xF2, 0xF3, 0xF4, 0xF5,  0xE6, 0xE8, 0xE3, 0xFE,  0xFB, 0xFD, 0xFF, 0xF9,  0xF8, 0xFC, 0xE0, 0xF1
,  0xC1, 0xC2, 0xD7, 0xC7,  0xC4, 0xC5, 0xD6, 0xDA,  0xC9, 0xCA, 0xCB, 0xCC,  0xCD, 0xCE, 0xCF, 0xD0
,  0xD2, 0xD3, 0xD4, 0xD5,  0xC6, 0xC8, 0xC3, 0xDE,  0xDB, 0xDD, 0xDF, 0xD9,  0xD8, 0xDC, 0xC0, 0xD1
};

static const u_char
recobe_k2z[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0, 0xA6, 0xCC
,     0,    0,    0,    0,     0, 0xA8, 0xB9,    0,     0,    0, 0xA0,    0,  0xB0,    0, 0xB7,    0
,     0,    0,    0, 0xB1,  0xCD,    0, 0xA9, 0xC0,     0, 0xC8, 0xCB,    0,  0xA5, 0xA2,    0,    0
,  0xC3,    0, 0xC1, 0xAD,  0xC2,    0,    0, 0xCA,     0, 0xA4,    0,    0,  0xCF, 0xCE,    0,    0

,  0xAF, 0xE1, 0xED, 0xB5,  0xF1, 0xD1, 0xF5, 0xFA,  0xD5, 0xBF, 0xAE, 0xAC,  0xBD, 0xBC, 0xA1, 0xAB
,  0xBB, 0xB4, 0xD3, 0xDF,  0xD4, 0xD2, 0xAA, 0xF3,  0xFD, 0xD9, 0xBA, 0xDE,  0xDD, 0xDA, 0xFE, 0xDB
,  0xD7, 0xC7, 0xFC, 0xFB,  0xE4, 0xE0, 0xF6, 0xE2,  0xF2, 0xEA, 0xEB, 0xE8,  0xEF, 0xEE, 0xEC, 0xC4
,  0xC5, 0x83, 0xC9, 0xE6,  0xC6, 0xF4, 0xE5, 0xE9,  0xA3, 0xF8, 0xE7, 0xFF,  0xD8, 0xD6, 0xF9, 0xDC
};

static const u_char
recobe_z2k[256] = /* iconv -f cp1252 -t cp850 | iconv -f cp866 */
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F

,     0,    0,    0,  '',     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0x9A,  '', 0xAD,  '',  0xB9, 0xAC, 0x8E,    0,  0x95, 0xA6,  '',  '',   '',  '',  '',  ''
,  0x9C,  '',    0,    0,   '',  '',    0, 0x9E,     0, 0x96,  '',  '',   '',  '',    0,  ''

,  0xA7, 0xB2, 0xB4, 0xB0,   '',  '',  '',  '',  0xA9,  '', 0xB7, 0xAA,  0x8F, 0xA4, 0xBD, 0xBC
,     0,  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  ''
,   '',  '',  '',    0,   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  ''
,     0,  '',  '',  '',   '',  '',  '',   00,   '',  '',  '',  '',   '',  '',  '',  ''
};

static const u_char
recobe_k2K[256] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21,  '', 0x23,  0x24, 0x25, 0x26,  '',  0x28, 0x29, 0x2A, 0x2B,   '', 0x2D,  '', 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39,  '',  '',   '', 0x3D,  '', 0x3F

,  0x40,  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  ''
,   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',  0x5C,  '', 0x5E, 0x5F
,   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  ''
,   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',  0x7C,  '',  '', 0x7F

,  0x80, 0x81, 0x82, 0x83,  0x84, 0x85, 0x86, 0x87,  0x88, 0x89, 0x8A, 0x8B,  0x8C, 0x8D, 0x8E, 0x8F
,  0x90, 0x91, 0x92, 0x93,  0x94, 0x95, 0x96, 0x97,  0x98, 0x99, 0x9A, 0x9B,  0x9C, 0x9D, 0x9E, 0x9F
,  0xA0, 0xA1, 0xA2,  '`',  0xA4, 0xA5, 0xA6, 0xA7,  0xA8, 0xA9, 0xAA, 0xAB,  0xAC, 0xAD, 0xAE, 0xAF
,  0xB0, 0xB1, 0xB2,  '~',  0xB4, 0xB5, 0xB6, 0xB7,  0xB8, 0xB9, 0xBA, 0xBB,  0xBC, 0xBD, 0xBE, 0xBF

,   '.',  'f',  ',',  'w',   'l',  't',  'a',  'u',   '[',  'b',  'q',  'r',   'k',  'v',  'y',  'j'
,   'g',  'z',  'h',  'c',   'n',  'e',  ';',  'd',   'm',  's',  'p',  'i',  '\'',  'o',  'x',  ']'
,   '>',  'F',  '<',  'W',   'L',  'T',  'A',  'U',   '{',  'B',  'Q',  'R',   'K',  'V',  'Y',  'J'
,   'G',  'Z',  'H',  'C',   'N',  'E',  ':',  'D',   'M',  'S',  'P',  'I',   '"',  'O',  'X',  '}'
};

static const u_char
recobe_K2k[] =
{  0x00, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0xFC, 0x23,  0x24, 0x25, 0x26, 0xDC,  0x28, 0x29, 0x2A, 0x2B,  0xC2, 0x2D, 0xC0, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0xF6, 0xD6,  0xE2, 0x3D, 0xE0, 0x3F

,  0x40, 0xE6, 0xE9, 0xF3,  0xF7, 0xF5, 0xE1, 0xF0,  0xF2, 0xFB, 0xEF, 0xEC,  0xE4, 0xF8, 0xF4, 0xFD
,  0xFA, 0xEA, 0xEB, 0xF9,  0xE5, 0xE7, 0xED, 0xE3,  0xFE, 0xEE, 0xF1, 0xC8,  0x5C, 0xDF, 0x5E, 0x5F
,  0xA3, 0xC6, 0xC9, 0xD3,  0xD7, 0xD5, 0xC1, 0xD0,  0xD2, 0xDB, 0xCF, 0xCC,  0xC4, 0xD8, 0xD4, 0xDD
,  0xDA, 0xCA, 0xCB, 0xD9,  0xC5, 0xC7, 0xCD, 0xC3,  0xDE, 0xCE, 0xD1, 0xE8,  0x7C, 0xFF, 0xB3, 0x7F

,  0x80, 0x81, 0x82, 0x83,  0x84, 0x85, 0x86, 0x87,  0x88, 0x89, 0x8A, 0x8B,  0x8C, 0x8D, 0x8E, 0x8F
,  0x90, 0x91, 0x92, 0x93,  0x94, 0x95, 0x96, 0x97,  0x98, 0x99, 0x9A, 0x9B,  0x9C, 0x9D, 0x9E, 0x9F
,  0xA0, 0xA1, 0xA2,    0,  0xA4, 0xA5, 0xA6, 0xA7,  0xA8, 0xA9, 0xAA, 0xAB,  0xAC, 0xAD, 0xAE, 0xAF
,  0xB0, 0xB1, 0xB2,    0,  0xB4, 0xB5, 0xB6, 0xB7,  0xB8, 0xB9, 0xBA, 0xBB,  0xBC, 0xBD, 0xBE, 0xBF

,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
};

static const u_char *
recobe_text_ic[RECOBE_TEXT_L][RECOBE_TEXT_L] =
{ {         0, recobe_a2i, recobe_a2k, recobe_a2w,          0,          0}
, {recobe_i2a,          0, recobe_i2k, recobe_i2w,          0,          0}
, {recobe_k2a, recobe_k2i,          0, recobe_k2w, recobe_k2z, recobe_k2K}
, {recobe_w2a, recobe_w2i, recobe_w2k,          0,          0,          0}
, {         0,          0, recobe_z2k,          0,          0,          0}
, {         0,          0, recobe_K2k,          0,          0,          0}
};

static const u_char
eta_a2i[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,  0xB0, 0xB1, 0xB2, 0xB3,  0xB4, 0xB5, 0xB6, 0xB7,  0xB8, 0xB9, 0xBA, 0xBB,  0xBC, 0xBD, 0xBE, 0xBF
,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0xA1, 0xF1, 0xA4, 0xF4,  0xA7, 0xF7, 0xAE, 0xFE,     0,    0,    0,    0,  0xF0,    0,    0, 0xA0
};

static const u_char
eta_i2a[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0xFF, 0xF0,    0,    0,  0xF2,    0,    0, 0xF4,     0,    0,    0,    0,     0,    0, 0xF6,    0
,  0x80, 0x81, 0x82, 0x83,  0x84, 0x85, 0x86, 0x87,  0x88, 0x89, 0x8A, 0x8B,  0x8C, 0x8D, 0x8E, 0x8F
,  0x90, 0x91, 0x92, 0x93,  0x94, 0x95, 0x96, 0x97,  0x98, 0x99, 0x9A, 0x9B,  0x9C, 0x9D, 0x9E, 0x9F
,  0xA0, 0xA1, 0xA2, 0xA3,  0xA4, 0xA5, 0xA6, 0xA7,  0xA8, 0xA9, 0xAA, 0xAB,  0xAC, 0xAD, 0xAE, 0xAF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0xFC, 0xF1,    0,    0,  0xF3,    0,    0, 0xF5,     0,    0,    0,    0,     0,    0, 0xF7,    0
};

static const u_char
eta_i2k[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0x9A, 0xB3,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0xE1, 0xE2, 0xF7, 0xE7,  0xE4, 0xE5, 0xF6, 0xFA,  0xE9, 0xEA, 0xEB, 0xEC,  0xED, 0xEE, 0xEF, 0xF0
,  0xF2, 0xF3, 0xF4, 0xF5,  0xE6, 0xE8, 0xE3, 0xFE,  0xFB, 0xFD, 0xFF, 0xF9,  0xF8, 0xFC, 0xE0, 0xF1
,  0xC1, 0xC2, 0xD7, 0xC7,  0xC4, 0xC5, 0xD6, 0xDA,  0xC9, 0xCA, 0xCB, 0xCC,  0xCD, 0xCE, 0xCF, 0xD0
,  0xD2, 0xD3, 0xD4, 0xD5,  0xC6, 0xC8, 0xC3, 0xDE,  0xDB, 0xDD, 0xDF, 0xD9,  0xD8, 0xDC, 0xC0, 0xD1
,     0, 0xA3,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
};

static const u_char
eta_k2i[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0, 0xA0,    0,     0,    0,    0,    0
,     0,    0,    0, 0xF1,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0, 0xA1,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0xEE, 0xD0, 0xD1, 0xE6,  0xD4, 0xD5, 0xE4, 0xD3,  0xE5, 0xD8, 0xD9, 0xDA,  0xDB, 0xDC, 0xDD, 0xDE
,  0xDF, 0xEF, 0xE0, 0xE1,  0xE2, 0xE3, 0xD6, 0xD2,  0xEC, 0xEB, 0xD7, 0xE8,  0xED, 0xE9, 0xE7, 0xEA
,  0xCE, 0xB0, 0xB1, 0xC6,  0xB4, 0xB5, 0xC4, 0xB3,  0xC5, 0xB8, 0xB9, 0xBA,  0xBB, 0xBC, 0xBD, 0xBE
,  0xBF, 0xCF, 0xC0, 0xC1,  0xC2, 0xC3, 0xB6, 0xB2,  0xCC, 0xCB, 0xB7, 0xC8,  0xCD, 0xC9, 0xC7, 0xCA
};

static const u_char
eta_a2k[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,  0xE1, 0xE2, 0xF7, 0xE7,  0xE4, 0xE5, 0xF6, 0xFA,  0xE9, 0xEA, 0xEB, 0xEC,  0xED, 0xEE, 0xEF, 0xF0
,  0xF2, 0xF3, 0xF4, 0xF5,  0xE6, 0xE8, 0xE3, 0xFE,  0xFB, 0xFD, 0xFF, 0xF9,  0xF8, 0xFC, 0xE0, 0xF1
,  0xC1, 0xC2, 0xD7, 0xC7,  0xC4, 0xC5, 0xD6, 0xDA,  0xC9, 0xCA, 0xCB, 0xCC,  0xCD, 0xCE, 0xCF, 0xD0
,  0x90, 0x91, 0x92, 0x81,  0x87, 0xB2, 0xB4, 0xA7,  0xA6, 0xB5, 0xA1, 0xA8,  0xAE, 0xAD, 0xAC, 0x83
,  0x84, 0x89, 0x88, 0x86,  0x80, 0x8A, 0xAF, 0xB0,  0xAB, 0xA5, 0xBB, 0xB8,  0xB1, 0xA0, 0xBE, 0xB9
,  0xBA, 0xB6, 0xB7, 0xAA,  0xA9, 0xA2, 0xA4, 0xBD,  0xBC, 0x85, 0x82, 0x8D,  0x8C, 0x8E, 0x8F, 0x8B
,  0xD2, 0xD3, 0xD4, 0xD5,  0xC6, 0xC8, 0xC3, 0xDE,  0xDB, 0xDD, 0xDF, 0xD9,  0xD8, 0xDC, 0xC0, 0xD1
,  0xB3, 0xA3,    0,    0,     0,    0,    0,    0,  0x9C, 0x95, 0x9E, 0x96,     0,    0, 0x94, 0x9A
};

static const u_char
eta_k2a[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,  0xC4, 0xB3, 0xDA, 0xBF,  0xC0, 0xD9, 0xC3, 0xB4,  0xC2, 0xC1, 0xC5, 0xDF,  0xDC, 0xDB, 0xDD, 0xDE
,  0xB0, 0xB1, 0xB2,    0,  0xFE, 0xF9, 0xFB,    0,     0,    0, 0xFF,    0,  0xF8,    0, 0xFA,    0
,  0xCD, 0xBA, 0xD5, 0xF1,  0xD6, 0xC9, 0xB8, 0xB7,  0xBB, 0xD4, 0xD3, 0xC8,  0xBE, 0xBD, 0xBC, 0xC6
,  0xC7, 0xCC, 0xB5, 0xF0,  0xB6, 0xB9, 0xD1, 0xD2,  0xCB, 0xCF, 0xD0, 0xCA,  0xD8, 0xD7, 0xCE,    0
,  0xEE, 0xA0, 0xA1, 0xE6,  0xA4, 0xA5, 0xE4, 0xA3,  0xE5, 0xA8, 0xA9, 0xAA,  0xAB, 0xAC, 0xAD, 0xAE
,  0xAF, 0xEF, 0xE0, 0xE1,  0xE2, 0xE3, 0xA6, 0xA2,  0xEC, 0xEB, 0xA7, 0xE8,  0xED, 0xE9, 0xE7, 0xEA
,  0x9E, 0x80, 0x81, 0x96,  0x84, 0x85, 0x94, 0x83,  0x95, 0x88, 0x89, 0x8A,  0x8B, 0x8C, 0x8D, 0x8E
,  0x8F, 0x9F, 0x90, 0x91,  0x92, 0x93, 0x86, 0x82,  0x9C, 0x9B, 0x87, 0x98,  0x9D, 0x99, 0x97, 0x9A
};

static const u_char
eta_a2w[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0xF0, 0xF1, 0xF2, 0xF3,  0xF4, 0xF5, 0xF6, 0xF7,  0xF8, 0xF9, 0xFA, 0xFB,  0xFC, 0xFD, 0xFE, 0xFF
,  0xA8, 0xB8, 0xAA, 0xBA,  0xAF, 0xBF, 0xA1, 0xA2,  0xB0,    0, 0xB7,    0,  0xB9, 0xA4,    0, 0xA0
};

static const u_char
eta_w2a[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0xFF, 0xF6, 0xF7,    0,  0xFD,    0,    0,    0,  0xF0,    0, 0xF2,    0,     0,    0,    0, 0xF4
,  0xF8,    0,    0,    0,     0,    0,    0, 0xFA,  0xF1, 0xFC, 0xF3,    0,     0,    0,    0, 0xF5
,  0x80, 0x81, 0x82, 0x83,  0x84, 0x85, 0x86, 0x87,  0x88, 0x89, 0x8A, 0x8B,  0x8C, 0x8D, 0x8E, 0x8F
,  0x90, 0x91, 0x92, 0x93,  0x94, 0x95, 0x96, 0x97,  0x98, 0x99, 0x9A, 0x9B,  0x9C, 0x9D, 0x9E, 0x9F
,  0xA0, 0xA1, 0xA2, 0xA3,  0xA4, 0xA5, 0xA6, 0xA7,  0xA8, 0xA9, 0xAA, 0xAB,  0xAC, 0xAD, 0xAE, 0xAF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
};

static const u_char
eta_i2w[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0xA0, 0xA8, 0x80, 0x81,  0xAA, 0xBD, 0xB2, 0xAF,  0xA3, 0x8A, 0x8C, 0x8E,  0x8D, 0xAD, 0xA1, 0x8F
,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0xF0, 0xF1, 0xF2, 0xF3,  0xF4, 0xF5, 0xF6, 0xF7,  0xF8, 0xF9, 0xFA, 0xFB,  0xFC, 0xFD, 0xFE, 0xFF
,  0xB9, 0xB8, 0x90, 0x83,  0xBA, 0xBE, 0xB3, 0xBF,  0xBC, 0x9A, 0x9C, 0x9E,  0x9D, 0xA7, 0xA2, 0x9F
};

static const u_char
eta_w2i[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,  0xA2, 0xA3,    0, 0xF3,     0,    0,    0,    0,     0,    0, 0xA9,    0,  0xAA, 0xAC, 0xAB, 0xAF
,  0xF2,    0,    0,    0,     0,    0,    0,    0,     0,    0, 0xF9,    0,  0xFA, 0xFC, 0xFB, 0xFF
,  0xA0, 0xAE, 0xFE, 0xA8,     0,    0,    0, 0xFD,  0xA1,    0, 0xA4,    0,     0, 0xAD,    0, 0xA7
,     0,    0, 0xA6, 0xF6,     0,    0,    0,    0,  0xF1, 0xF0, 0xF4,    0,  0xF8, 0xA5, 0xF5, 0xF7
,  0xB0, 0xB1, 0xB2, 0xB3,  0xB4, 0xB5, 0xB6, 0xB7,  0xB8, 0xB9, 0xBA, 0xBB,  0xBC, 0xBD, 0xBE, 0xBF
,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
};

static const u_char
eta_k2w[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0, 0xA0,    0,  0xB0,    0, 0xB7,    0
,     0,    0,    0, 0xB8,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0, 0xA8,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0, 0xA9
,  0xFE, 0xE0, 0xE1, 0xF6,  0xE4, 0xE5, 0xF4, 0xE3,  0xF5, 0xE8, 0xE9, 0xEA,  0xEB, 0xEC, 0xED, 0xEE
,  0xEF, 0xFF, 0xF0, 0xF1,  0xF2, 0xF3, 0xE6, 0xE2,  0xFC, 0xFB, 0xE7, 0xF8,  0xFD, 0xF9, 0xF7, 0xFA
,  0xDE, 0xC0, 0xC1, 0xD6,  0xC4, 0xC5, 0xD4, 0xC3,  0xD5, 0xC8, 0xC9, 0xCA,  0xCB, 0xCC, 0xCD, 0xCE
,  0xCF, 0xDF, 0xD0, 0xD1,  0xD2, 0xD3, 0xC6, 0xC2,  0xDC, 0xDB, 0xC7, 0xD8,  0xDD, 0xD9, 0xD7, 0xDA
};

static const u_char
eta_w2k[] =
{     0, 0x01, 0x02, 0x03,  0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x0A, 0x0B,  0x0C, 0x0D, 0x0E, 0x0F
,  0x10, 0x11, 0x12, 0x13,  0x14, 0x15, 0x16, 0x17,  0x18, 0x19, 0x1A, 0x1B,  0x1C, 0x1D, 0x1E, 0x1F
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F
,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E, 0x7F
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0x9A,    0,    0,    0,     0,    0,    0,    0,  0xB3, 0xBF,    0,    0,     0,    0,    0,    0
,  0x9C,    0,    0,    0,     0,    0,    0, 0x9E,  0xA3,    0,    0,    0,     0,    0,    0,    0
,  0xE1, 0xE2, 0xF7, 0xE7,  0xE4, 0xE5, 0xF6, 0xFA,  0xE9, 0xEA, 0xEB, 0xEC,  0xED, 0xEE, 0xEF, 0xF0
,  0xF2, 0xF3, 0xF4, 0xF5,  0xE6, 0xE8, 0xE3, 0xFE,  0xFB, 0xFD, 0xFF, 0xF9,  0xF8, 0xFC, 0xE0, 0xF1
,  0xC1, 0xC2, 0xD7, 0xC7,  0xC4, 0xC5, 0xD6, 0xDA,  0xC9, 0xCA, 0xCB, 0xCC,  0xCD, 0xCE, 0xCF, 0xD0
,  0xD2, 0xD3, 0xD4, 0xD5,  0xC6, 0xC8, 0xC3, 0xDE,  0xDB, 0xDD, 0xDF, 0xD9,  0xD8, 0xDC, 0xC0, 0xD1
};

static const u_char *
recobe_text_et[RECOBE_TEXT_L][RECOBE_TEXT_L] =
{ {      0, eta_a2i, eta_a2k, eta_a2w, 0, 0}
, {eta_i2a,       0, eta_i2k, eta_i2w, 0, 0}
, {eta_k2a, eta_k2i,       0, eta_k2w, 0, 0}
, {eta_w2a, eta_w2i, eta_w2k,       0, 0, 0}
, {      0,       0,       0,       0, 0, 0}
, {      0,       0,       0,       0, 0, 0}
};

static const char
recobe_text_nl[] =
{     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0

,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0

,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0

,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
};

static const char
recobe_text_kp[] =
{     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,  0x20, 0x21, 0x22, 0x23,  0x24, 0x25, 0x26, 0x27,  0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F
,  0x30, 0x31, 0x32, 0x33,  0x34, 0x35, 0x36, 0x37,  0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x3F

,  0x40, 0x41, 0x42, 0x43,  0x44, 0x45, 0x46, 0x47,  0x48, 0x49, 0x4A, 0x4B,  0x4C, 0x4D, 0x4E, 0x4F
,  0x50, 0x51, 0x52, 0x53,  0x54, 0x55, 0x56, 0x57,  0x58, 0x59, 0x5A, 0x5B,  0x5C, 0x5D, 0x5E, 0x5F
,  0x60, 0x61, 0x62, 0x63,  0x64, 0x65, 0x66, 0x67,  0x68, 0x69, 0x6A, 0x6B,  0x6C, 0x6D, 0x6E, 0x6F
,  0x70, 0x71, 0x72, 0x73,  0x74, 0x75, 0x76, 0x77,  0x78, 0x79, 0x7A, 0x7B,  0x7C, 0x7D, 0x7E,    0

,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0, 0xA3,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
,     0,    0,    0, 0xB3,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0

,  0xC0, 0xC1, 0xC2, 0xC3,  0xC4, 0xC5, 0xC6, 0xC7,  0xC8, 0xC9, 0xCA, 0xCB,  0xCC, 0xCD, 0xCE, 0xCF
,  0xD0, 0xD1, 0xD2, 0xD3,  0xD4, 0xD5, 0xD6, 0xD7,  0xD8, 0xD9, 0xDA, 0xDB,  0xDC, 0xDD, 0xDE, 0xDF
,  0xE0, 0xE1, 0xE2, 0xE3,  0xE4, 0xE5, 0xE6, 0xE7,  0xE8, 0xE9, 0xEA, 0xEB,  0xEC, 0xED, 0xEE, 0xEF
,  0xF0, 0xF1, 0xF2, 0xF3,  0xF4, 0xF5, 0xF6, 0xF7,  0xF8, 0xF9, 0xFA, 0xFB,  0xFC, 0xFD, 0xFE, 0xFF
};

static const recobe_c_tnm
recobe_text_nm[RECOBE_TEXT_L] =
{ { 'a', "ALT" , recobe_text_nl}
, { 'i', "ISO" , recobe_text_nl}
, { 'k', "KOI8", recobe_text_kp}
, { 'w', "WIN" , recobe_text_nl}
, { 'z', "ZIP" , recobe_text_nl}
, { 'K', "KBD" , recobe_text_nl}
};

#define RECOBETABLES recobe_text_ic
//#define RECOBETABLES recobe_text_et

static u_char *
recobe_text_get(const char *name) {
    u_char *t = NULL;
    int i, j;

    if  (  !!name[0]
        && !!name[1]
        && (0 <= (i = recobe_collection.tst(name[0], RECOBE_TEXT)))
        && (0 <= (j = recobe_collection.tst(name[1], RECOBE_TEXT)))
        ) {
        if  (!!recobe_text_ic[i][j] && !!(t = malloc(256))) bcopy(recobe_text_ic[i][j], t, 256);
    }
    return(t);
}

static u_char *
recobe_text_ind(int f, int t) {
    u_char *s = NULL;

    if  ((f < RECOBE_TEXT_L) && (t < RECOBE_TEXT_L) && !!recobe_text_ic[f][t] && (s = malloc(256))) {
        bcopy(recobe_text_ic[f][t], s, 256);
    }
    return(s);
}

const recobe_c_tdm
/*****************************************************************************************************
 *****************************************************************************************************/
recobe_text_dm =                                                                                 /****
 *****************************************************************************************************
 *****************************************************************************************************/
{ RECOBE_TEXT_L
, "TEXT"
, "recobe_"
, recobe_text_nm
, recobe_text_get
, recobe_text_ind
};
