/*-
 * Copyright (C)2014 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2014 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n\
@(#)V.M "VERS;
static const char rcsid[] = "$Id: recobe.c,v 1.5 2014/08/31 06:34:45 babolo Exp $";
#endif /* not lint */

#define BLIN_COMPAT  3
#define MIFE_COMPAT  4
#define Bpars_COMPAT 3

#include <sys/types.h>
#include <sys/stat.h>
#include <sysexits.h>
#include <strings.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include "recobe.h"

#define RECOBE_PRINTABLE 0x000100

static void
p256(const char *temlate, char from, char to, u_char domain, const u_char array[256]) {
    int i, k;

    if  (!temlate) {
        printf("{  ");
    } else {
        printf("static const u_char %s%c2%c[] =\n{  ", temlate, from, to);
    }
    for (k = 0; k < 256; ++k) {
        if  (k) {
            if  (!(k & 15)) printf("\n");
            printf(", ");
            if  (!(k & 3)) printf(" ");
        }
        if  (  (0 > (i = recobe_collection.tst(to, domain)))
            || !recobe_collection.dm[domain]->nm[i].p
            ) {
            printf("0x%02X", array[k]);
        } else if (!array[k]) {
            printf("   0");
        } else if (  !(blin_internal_flags & RECOBE_PRINTABLE)
                  || !recobe_collection.dm[domain]->nm[i].p[array[k]]
                  ) {
            printf("0x%02X", array[k]);
        } else switch (recobe_collection.dm[domain]->nm[i].p[array[k]]) {
        case '\\': case '\'':
            printf("'\\%c'", recobe_collection.dm[domain]->nm[i].p[array[k]]);
            break;
        default:
            printf(" '%c'", recobe_collection.dm[domain]->nm[i].p[array[k]]);
    }   }
    printf("\n};\n");
}

static int
TestOrRevert(const char *p) {
    int g, i, j, k, domain;
    u_char *s, *r;

    domain = (!*p) ? 0 : (*p - '0');
    if  (recobe_collection.mw <= domain) domain = 0;
    for (g = (!domain ? 1 : domain); g < (!domain ? recobe_collection.mw : domain + 1); ++g) {
        for (i = 0; i < recobe_collection.dm[g]->cw; ++i) {
            for (j = 0; j < recobe_collection.dm[g]->cw; ++j) {
                if  (!!(s = recobe_collection.ind(g, i, j))) {
                    if  (!!(r = recobe_collection.ind(g, j, i))) {
                        for (k = 0; k < 256; ++k) {
                            if  (  (r[s[k]] != k)
                                && (  (!!r[s[k]] && !!s[k]) || (g != RECOBE_TEXT))
                                ) {
                                printf( "%c2%c[0x%02X]=0x%02X, %c2%c[0x%02X]=0x%02X\n"
                                      , recobe_collection.dm[g]->nm[i].n
                                      , recobe_collection.dm[g]->nm[j].n
                                      , k
                                      , s[k]
                                      , recobe_collection.dm[g]->nm[j].n
                                      , recobe_collection.dm[g]->nm[i].n
                                      , s[k]
                                      , r[s[k]]
                                      )
                                ;
                        }   }
                        free(r);
                        r = NULL;
                    } else {
                        u_char o[256];

                        bzero(o, 256);
                        for (k = 0; k < 256; ++k) o[s[k]] = k;
                        p256( recobe_collection.dm[g]->pp
                            , recobe_collection.dm[g]->nm[j].n
                            , recobe_collection.dm[g]->nm[i].n
                            , g
                            , o
                            )
                        ;
                    }
                    free(s);
                    s = NULL;
    }   }   }   }
    exit(EX_OK);
}

static int
PrintAndMake(const char *p) {
    u_char *t, *f, o[256];
    int k, i, j, g, q;

    for (g = 1; g < recobe_collection.mw; ++g) {
        for (i = 0; i < recobe_collection.dm[g]->cw; ++i) {
            if  (!p[0] || (p[0] == recobe_collection.dm[g]->nm[i].n)) {
                for (j = 0; j < recobe_collection.dm[g]->cw; ++j) {
                    if  (!p[0] || !p[1] || (p[1] == recobe_collection.dm[g]->nm[j].n)) {
                        if  (!!(t = recobe_collection.ind(g, i, j))) {
                            if  ('-' == p[2]) {
                                write(fileno(stdout), t, 256);
                            } else {
                                p256( recobe_collection.dm[g]->pp
                                    , recobe_collection.dm[g]->nm[i].n
                                    , recobe_collection.dm[g]->nm[j].n
                                    , g
                                    , t
                                    )
                                ;
                            }
                            free(t);
                            t = NULL;
                        } else if (!!p[0] && !!p[1] && !!(f = recobe_collection.ind(g, j, i))) {
                            for (k = 0; k < 256; ++k) o[k] = 0;
                            for (k = 0; k < 256; ++k) o[f[k]] = k;
                            if  ('-' == p[2]) {
                                write(fileno(stdout), o, 256);
                            } else {
                                p256( recobe_collection.dm[g]->pp
                                    , recobe_collection.dm[g]->nm[i].n
                                    , recobe_collection.dm[g]->nm[j].n
                                    , g
                                    , o
                                    )
                                ;
                            }
                            free(f);
                            f = NULL;
                        } else if (!!p[0] && !!p[1]) {
                            for (q = 0; q < recobe_collection.dm[g]->cw; ++q) {
                                if  ((i != q) && (q != j) && !!(f = recobe_collection.ind(g, i, q))) {
                                    if  (!!(t = recobe_collection.ind(g, q, j))) {
                                        for (k = 0; k < 256; ++k) o[k] = t[f[k]];
                                        if  ('-' == p[2]) {
                                            write(fileno(stdout), o, 256);
                                        } else {
                                        printf( "/* %c=>%c=>%c */\n"
                                              , recobe_collection.dm[g]->nm[i].n
                                              , recobe_collection.dm[g]->nm[q].n
                                              , recobe_collection.dm[g]->nm[j].n
                                              )
                                        ;
                                            p256( recobe_collection.dm[g]->pp
                                                , recobe_collection.dm[g]->nm[i].n
                                                , recobe_collection.dm[g]->nm[j].n
                                                , g
                                                , o
                                                )
                                            ;
                                        }
                                        free(t);
                                        t = NULL;
                                    }
                                    free(f);
                                    f = NULL;
    }   }   }   }   }   }   }   }
    exit(EX_OK);
}

static int
PrintNoRecod(const char *p) {
    u_char o[256];
    int k;

    for (k = 0; k < 256; ++k) o[k] = k;
    if  (!p[0]) {
        write(fileno(stdout), o, 256);
    } else {
        p256( p[0] && p[1] && p[2] ? &p[2] : "X"
            , p[0]
            , p[0] && p[1] ? p[1] : 'x'
            , RECOBE_TEXT
            , o
            )
        ;
    }
    exit(EX_OK);
}

static int
G711DiffConv(const char *p) {
    u_char *s;
    int i;

    if  (!*p || !strcmp(p, "A")) {
        for (i = 0; i < 256; ++i) {
            if  (recobe_G711A(recobe_deG711A[i]) != i) {
                printf( "A %02X %04X %02X\n"
                      , i, recobe_deG711A[i] & 0xFFFF, recobe_G711A(recobe_deG711A[i])
                      )
                ;
    }   }   }
    if  (!*p || !strcmp(p, "u")) {
        for (i = 0; i < 256; ++i) {
            if  (recobe_G711u(recobe_deG711u[i]) != i) {
                printf( "u %02X %04X %02X\n"
                      , i, recobe_deG711u[i] & 0xFFFF, recobe_G711u(recobe_deG711u[i])
                      )
                ;
    }   }   }
    if  (!*p || !strncmp(p, "Au", 2)) {
        s = recobe_collection.get("Au", RECOBE_G711);
        for (i = 0; i < 256; ++i) {
            if  (s[i] != recobe_G711u(recobe_deG711A[i])) {
                printf( "u[A[%02X]=%04X]=%02X <> A2u[%02X]=%02X\n"
                      , i
                      , recobe_deG711A[i] & 0xFFFF
                      , recobe_G711u(recobe_deG711A[i])
                      , i
                      , s[i]
                      )
                ;
        }   }
        free(s);
        s = NULL;
    }
    if  (!*p || !strncmp(p, "uA", 2)) {
        s = recobe_collection.get("uA", RECOBE_G711);
        for (i = 0; i < 256; ++i) {
            if  (s[i] != recobe_G711A(recobe_deG711u[i])) {
                printf( "A[u[%02X]=%04X]=%02X <> u2A[%02X]=%02X\n"
                      , i
                      , recobe_deG711u[i] & 0xFFFF
                      , recobe_G711A(recobe_deG711u[i])
                      , i
                      , s[i]
                      )
                ;
        }   }
        free(s);
        s = NULL;
    }
    exit(EX_OK);
}

static int
PrintFilling(const char *p) {
    u_char  *s, o[256];
    int g, i, j, k, domain;

    domain = (!*p) ? 0 : (*p - '0');
    if  (recobe_collection.mw <= domain) domain = 0;
    for (g = (!domain ? 1 : domain); g < (!domain ? recobe_collection.mw : domain + 1); ++g) {
        for (i = 0; i < recobe_collection.dm[g]->cw; ++i) {
            for (j = 0; j < recobe_collection.dm[g]->cw; ++j) {
                if  (!!(s = recobe_collection.ind(g, i, j))) {
                    for (k = 0; k < 256; ++k) o[k] = 0;
                    for (k = 0; k < 256; ++k) o[s[k]]++;
                    free(s);
                    s = NULL;
                    for (k = 0; k < 256; ++k) {
                        if  (1 != o[k]) {
                            printf(" g %d %c%c[0x%02X]=%d\n"
                                  , g
                                  , recobe_collection.dm[g]->nm[i].n
                                  , recobe_collection.dm[g]->nm[j].n
                                  , k
                                  , o[k]
                                  )
                            ;
    }   }   }   }   }   }
    exit(EX_OK);
}

static int
InterMediaTo(const char *p) {
    u_char *s, *t, *f;
    int k, i, j, g, q;

    for (g = 1; g < recobe_collection.mw; ++g) {
        for (i = 0; i < recobe_collection.dm[g]->cw; ++i) {
            if  (!p[0] || (p[0] == recobe_collection.dm[g]->nm[i].n)) {
                for (j = 0; j < recobe_collection.dm[g]->cw; ++j) {
                    if  (!p[0] || !p[1] || (p[1] == recobe_collection.dm[g]->nm[j].n)) {
                        if  (!!(s = recobe_collection.ind(g, i, j))) {
                            for (q = 0; q < recobe_collection.dm[g]->cw; ++q) {
                                if  ((i != q) && (q != j) && !!(f = recobe_collection.ind(g, i, q))) {
                                    if  (!!(t = recobe_collection.ind(g, q, j))) {
                                        for (k = 0; k < 256; ++k) {
                                            if  (s[k] != t[f[k]]) {
                                                printf( "0x%02x: %c%c=0x%02x <"
                                                        "> %c%c%c=(0x%02x)0x%02x\n"
                                                      , k
                                                      , recobe_collection.dm[g]->nm[i].n
                                                      , recobe_collection.dm[g]->nm[j].n
                                                      , s[k]
                                                      , recobe_collection.dm[g]->nm[i].n
                                                      , recobe_collection.dm[g]->nm[q].n
                                                      , recobe_collection.dm[g]->nm[j].n
                                                      , f[k]
                                                      , t[f[k]]
                                                      )
                                                ;
                                        }   }
                                        free(t);
                                        t = NULL;
                                    }
                                    free(f);
                                    f = NULL;
                            }   }
                            free(s);
                            s = NULL;
    }   }   }   }   }   }
    exit(EX_OK);
}

static int
PrintRevName(const char *p) {
    u_char o[256];
    int g, k, domain;

    domain = (!*p) ? 0 : (*p - '0');
    for (g = (!domain ? 1 : domain); g < (!domain ? recobe_collection.mw : domain + 1); ++g) {
        for (k = 0; k < 256; ++k) o[k] = 0xFF;
        for (k = 0; k < recobe_collection.dm[g]->cw; ++k) {
            o[recobe_collection.dm[g]->nm[k].n & 0xFF] = k;
        }
        p256(NULL, 0, 0, RECOBE_NOTABLE, o);
    }
    exit(EX_OK);
}

static void
usage(int ex) {
    int g, i, j, k;
    u_char *t;

    fprintf(stderr, "recobe " VERS " @BABOLO " DATE "\nUsage: recobe [-options] -c{");
    k = 0;
    for (g = 1; g < recobe_collection.mw; ++g) {
        for (i = 0; i < recobe_collection.dm[g]->cw; ++i) {
            for (j = 0; j < recobe_collection.dm[g]->cw; ++j) {
                if  (!!(t = recobe_collection.ind(g, i, j))) {
                    if  (k++) fprintf(stderr, "|");
                    fprintf( stderr, "%c%c"
                           , recobe_collection.dm[g]->nm[i].n
                           , recobe_collection.dm[g]->nm[j].n
                           )
                    ;
                    free(t);
                    t = NULL;
    }   }   }   }
    fprintf(stderr, "}[ infile_or_string[ oufile]]\n");
    for (g = 1; g < recobe_collection.mw; ++g) {
        for (i = 0; i < recobe_collection.dm[g]->cw; ++i) {
            for (j = 0; j < recobe_collection.dm[g]->cw; ++j) {
                if  (!!(t = recobe_collection.ind(g, i, j))) {
                    fprintf( stderr
                           , "   -c %c%c           from %s to %s (%u%u)\n"
                           , recobe_collection.dm[g]->nm[i].n
                           , recobe_collection.dm[g]->nm[j].n
                           , recobe_collection.dm[g]->nm[i].s
                           , recobe_collection.dm[g]->nm[j].s
                           , i, j
                           )
                    ;
                    free(t);
                    t = NULL;
    }   }   }   }
    fprintf( stderr
           , "   -E -X XX        use XX as env name for opt's arg\n"
             "   -q              silent\n"
             "   -s code         control code\n"
             "   -v[v[v...]]     verbose\n"
             "   -z TEXT         parce TEXT as arguments continuation\n"
             "   -Z TEXT         parce TEXT as arguments continuation with env substitution\n"
             "   -= mode         internal diagnostic\n"
             "      mode: 0             blin_ctl(BLIN_CTL_DUMP)\n"
             "            1             babolo_optest(OptionsString)\n"
             "            2             babolo_optext(OptionsDescriptor)\n"
             "            3             babolo_dumpopts(OptionsDescriptor)\n"
             "            4[X]          inconsistent reverse, X - domain (1 TEXT, 2 G711)\n"
             "            5[Y[Z]]       print table(s); Y, Z code names\n"
             "            6[A[B[name]]] print norecoding table nameA2B[]\n"
             "            7{u|A|Au|uA}  inconsistent G711u, G711A, A=>u, u=>a\n"
             "            8[X]          count, X - domain\n"
             "            9[Y[Z]]       test intermediators; Y, Z code names\n"
             "   -?              this text\n"
           )
    ;
    exit(ex);
}

int
main(int argc, char **argv) {
    const char *o, *Tflag = NULL, *tflag = NULL, *cflag = NULL;
    const char *flar = "c:pqs:t:v=:?";
    FILE *in = stdin, *ou = stdout;
    int c, i, code, ex = EX_OK;
    u_char *table = NULL;
    babolo_opts *bos;
    ssize_t j;
    u_char *z;

    blin_ctl(BLIN_CTL_FSET, BLIN_BIT0);
    if  (!(bos = babolo_openopts(Bpars_ESYM | Bpars_SYME, 'Z'))) {
        ifBLIN_QW0("openopts");
        ex = EX_SOFTWARE;
        goto out;
    }
    if  (!!babolo_setopts(bos, Bpars_NONU, argc, argv, flar)) {
        ifBLIN_QW0("setopts");
        ex = EX_SOFTWARE;
        goto out;
    }
    code = 0;
    for (tflag = 0;;) {
        c = babolo_getopts(bos);
        if  (!c) break;
        switch (c) {
        case 'c': cflag = babolo_getoptsarg(bos);
                ; break;
        case 'p': blin_internal_flags |= RECOBE_PRINTABLE;
                ; break;
        case 'q': blin_ctl(BLIN_CTL_FRST, BLIN_MASK);
                ; break;
        case 's': code = babolo_getoptsnum(bos, 0);
                ; break;
        case 't': tflag = babolo_getoptsarg(bos);
                ; break;
        case 'v': BLIN_VERBOSE(blin_internal_flags);
                ; break;
        case '=': switch((Tflag = babolo_getoptsarg(bos))[0]) {
                  case '0': blin_ctl(BLIN_CTL_DUMP);      exit(EX_OK);
                  case '1': babolo_optest(flar);          exit(EX_OK);
                  case '2': babolo_optext(bos);           exit(EX_OK);
                  case '3': babolo_dumpopts(bos);         exit(EX_OK);
                  }
                ; break;
        case '?': usage(EX_OK);
        default : ifBLIN_QX0("Illegal flag -%c", bos->c);
                ; usage(EX_USAGE);
    }   }
    if  (Tflag) {
        switch (Tflag[0]) {
        case '4': ex = TestOrRevert(++Tflag); break;
        case '5': ex = PrintAndMake(++Tflag); break;
        case '6': ex = PrintNoRecod(++Tflag); break;
        case '7': ex = G711DiffConv(++Tflag); break;
        case '8': ex = PrintFilling(++Tflag); break;
        case '9': ex = InterMediaTo(++Tflag); break;
        case 'X': ex = PrintRevName(++Tflag); break;
        }
    } else if (!!code) {
        j = 0;
        if  (!(z = babolo_hextrin(babolo_getargs(bos), &j))) {
            ifBLIN_QW0("babolo_hextrin");
            ex = EX_IOERR;
            goto out;
        }
        ifBLIN_QO1 blin_dumb(1, z, j);
        if  (0 > code) j = -j;
        switch ((0 > code) ? -code : code) {
        case 1: printf("csum1 = %02X\n", recobe_csum1(z, j));
                break;
        case 2: printf("crc24 = %06X\n", recobe_crc24(z, j));
                break;
        case 3: printf("crc32 = %08X\n", recobe_crc32(z, j));
                break;
        default:
                ifBLIN_QX0("Illegal code %d", code);
        }
    } else {
        if  ((o = babolo_getargs(bos)) && !(in = fopen(o, "r"))) {
            ifBLIN_QW0("fopen in %s", o);
            ex = EX_IOERR;
            goto out;
        }
        if  ((o = babolo_getargs(bos)) && !(ou = fopen(o, "w"))) {
            ifBLIN_QW0("fopen ou %s", o);
            ex = EX_IOERR;
            goto out;
        }
        if  (!!cflag) {
            if  (!(table = recobe_collection.get(cflag, 0))) {
                ifBLIN_QX0("No recoding %s", argv[1]);
                ex = EX_USAGE;
                usage(ex);
            }
            while (0 <= (i = getc(in))) {
                i &= 0xFF;
                j = table[i];
                if  (i & !j) j = '?';
                putc(j, ou);
            }
            free(table);
        } else if (!!tflag) {
            table = malloc(256);
            for (i = 0; i < 256; ++i) {
                table[i] = getc(in);
            }
            p256(tflag + 2, tflag[0], tflag[1], RECOBE_TEXT, table);
        }
        fclose(in);
        fclose(ou);
    }
out:
    exit(ex);
}
